package net.sf.itcb.common.portlet.vaadin;

import javax.annotation.Resource;
import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.EventRequest;
import javax.portlet.EventResponse;
import javax.portlet.PortletMode;
import javax.portlet.PortletModeException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletURL;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.ResourceRequest;
import javax.portlet.ResourceResponse;
import javax.portlet.WindowState;
import javax.portlet.WindowStateException;

import net.sf.itcb.common.business.exceptions.BusinessItcbException;
import net.sf.itcb.common.portlet.util.PortalAdapterUtil;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Configurable;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;

import com.vaadin.Application;
import com.vaadin.terminal.gwt.server.PortletApplicationContext2;
import com.vaadin.terminal.gwt.server.PortletApplicationContext2.PortletListener;
import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Window;
import com.vaadin.ui.Window.CloseEvent;
import com.vaadin.ui.Window.Notification;


/**
 * this abstract class is the base vaadin application portlet used by all the vaadin application
 * It manages common treatments :
 * <ul>
 * <li>user storage in a spring Context {@link org.springframework.security.core.context.SecurityContextHolder}</li>
 * <li>local storage in a spring Context {@link org.springframework.context.i18n.LocaleContextHolder}</li> 
 * <li>user storage in MDC context in order to be logged {@link org.slf4j.MDC}</li>
 * <li>create the application and all the views (VIEW, EDIT, HELP) when the application is run inside a portal or directly in an application server as a servlet</li>
 * <li>delegates the views contents to child classes</li>
 * </ul>
 * 
 * TODO (Feature request  3155816): Request in an portal free mode (running in Tomcat as servlet) is ok but 2 things has to be done :
 * <ul>
 * <li>Request are now RenderRequest but it has to be also possible to use HttpServletRequest</li>
 * <li>Users have to be mocked when we are in Servlet mode</li>
 * <li>Edit View and Help mode have to be displayed with a tabs screen instead of the 3 actual buttons.</li>
 * </ul>
 * In child class and all the other classes : Services, DAO... The user will be available by calling the spring context : SecurityContextHolder.getContext().getAuthentication().
 * 
 * @author Pierre Le Roux
 */
@Configurable(preConstruction = true)
public abstract class AbstractItcbPortletApplication extends Application {
		
	private static final long serialVersionUID = 1L;
	
	protected final Logger log = LoggerFactory.getLogger(getClass());
	
	protected Window mainWindow;
	
	protected String preferencesURL;
	
	@Resource(name="portalAdapterUtil")
	protected PortalAdapterUtil portalAdaptaterUtil;
	
	@Resource(name="viewModePageMappingProcessor")
	protected PageMappingProcessor viewModePageMappingProcessor;
	
	@Resource(name="editModePageMappingProcessor")
	protected PageMappingProcessor editModePageMappingProcessor;
	
	@Resource(name="editModePageMappingProcessor")
	protected PageMappingProcessor helpModePageMappingProcessor;
	
	protected PageMappingProcessor currentPageMappingProcessor;
	
	protected boolean hasToCloseApplicationOnExit = false;
	
	public void init() {
		
		mainWindow = new Window(getWindowName());
		setMainWindow(mainWindow);
		
		// The portlet is launched in a portal
		if (getContext() instanceof PortletApplicationContext2) {
			PortletApplicationContext2 ctx =
				(PortletApplicationContext2) getContext();
			ctx.addPortletListener(this, new CommonPortletListener());
			
			
		}
		// The portlet is launched as a webapplication in a web app
		else {
			try {
				// TODO Add mocked request and response with spring configuration instead of null values
				// For now, portlet is ok outside a portal if the portlet doesn't use request and response objects
				//getMainWindow().addComponent(new PortletDevView(getPortletModeView(null, null), getPortletModeEdit(null, null), getPortletModeHelp(null,null)));
			} catch (Exception e) {
				// TODO technical errors should not be shown at screen in production mode.
				// In development mode, it's interesting to keep it.
				getMainWindow().showNotification(
						e.getMessage(), Notification.TYPE_ERROR_MESSAGE);
				e.printStackTrace();;
			}
		}
		
		mainWindow.addListener(new Window.CloseListener() {
			private static final long serialVersionUID = 1L;

			@Override
		    public void windowClose(CloseEvent poEvent)
			{
				initContextEvent();
				log.debug("Closing Application window");
				
				// Terminaison de l'application
				if(hasToCloseApplicationOnExit) {
					
					AbstractItcbPortletApplication.this.close();
				}
		    }
		});

	}
	
	private void initContext(PortletRequest request) {
		LocaleContextHolder.setLocale(this.getLocale());
		portalAdaptaterUtil.storeUserInContext(request);	
		if(SecurityContextHolder.getContext() != null && SecurityContextHolder.getContext().getAuthentication() != null) {
			
			this.setUser(SecurityContextHolder.getContext().getAuthentication());
		}
		else {
			portalAdaptaterUtil.storeLoggedUserInContext(null);
		}
		
	}
	
	private void initContextEvent() {
		LocaleContextHolder.setLocale(this.getLocale());
		if(this.getUser() != null) {
			portalAdaptaterUtil.storeLoggedUserInContext((Authentication)this.getUser());
		}
		else {
			portalAdaptaterUtil.storeLoggedUserInContext(null);
		}

	}
	
	public void setHasToCloseApplicationOnExit(boolean hasToCloseApplicationOnExit) {
		this.hasToCloseApplicationOnExit = hasToCloseApplicationOnExit;
	}

	protected abstract String getWindowName();
	
	
	private class CommonPortletListener implements PortletListener {

		private static final long serialVersionUID = 1L;

		public void handleActionRequest(
			ActionRequest request, ActionResponse response, Window window) {
			initContextEvent();
			log.debug("Logging after initUserContext");
		}

		public void handleRenderRequest(
			RenderRequest request, RenderResponse response, Window window) {
			if (preferencesURL == null) {
				PortletURL url = response.createActionURL();
				try {
					url.setPortletMode(PortletMode.EDIT);
					url.setWindowState(WindowState.MAXIMIZED);
					preferencesURL = url.toString();
				}
				catch (PortletModeException e) {
					// TODO technical errors should not be shown at screen in production mode.
					// In development mode, it's interesting to keep it.
					log.warn(e.getMessage());
				}
				catch (WindowStateException e) {
					// TODO technical errors should not be shown at screen in production mode.
					// In development mode, it's interesting to keep it.
					log.warn(e.getMessage());
				}
			}

			mainWindow.removeAllComponents();
			initContext(request);
			
			// check mode in request and act accordingly
			try {
				if (PortletMode.VIEW.equals(request.getPortletMode())) {
					currentPageMappingProcessor = viewModePageMappingProcessor;
				}
				else if (PortletMode.EDIT.equals(request.getPortletMode())) {
					currentPageMappingProcessor = editModePageMappingProcessor;
				}
				else if (PortletMode.HELP.equals(request.getPortletMode())) {
					currentPageMappingProcessor = helpModePageMappingProcessor;
				}
				if(currentPageMappingProcessor == null) {
					log.error("the "+ request.getPortletMode() +" associated mapping variable has to be set in spring context. The bean name has to be : " + request.getPortletMode() + "ModePageMappingProcessor");
					return;
				}
				currentPageMappingProcessor.setRequest(request);
				currentPageMappingProcessor.setPanel(getPanel(request.getPortletMode()));
				currentPageMappingProcessor.setApplication(AbstractItcbPortletApplication.this);
				if(currentPageMappingProcessor.isAutomaticDisplay()) {
					currentPageMappingProcessor.displayDefaultPage();
				}
			} catch (BusinessItcbException bie) {
				getMainWindow().showNotification(
						bie.getLocalizedMessage(), Notification.TYPE_WARNING_MESSAGE);
			}
			catch (Exception e) {
				// TODO technical errors should not be shown at screen in production mode.
				// In development mode, it's interesting to keep it.
				//getMainWindow().showNotification(
				//		e.getMessage(), Notification.TYPE_ERROR_MESSAGE);
				e.printStackTrace();
			}
			

		}

		/**
		 * Retrieves the panel in which the content will be displayed
		 * If the PortletApplication implements {@link PanelSpecificPortlet}, 
		 * it will display content in the specific panel depending on the portletMode
		 * @param portletMode the portlet mode
		 * @return the panel in which the content will be written
		 */
		protected Panel getPanel(PortletMode portletMode) {
			if(this instanceof PanelSpecificPortlet) {
				if(PortletMode.VIEW.equals(portletMode)) {
					return ((PanelSpecificPortlet)this).getPanelView();
				}
				else if(PortletMode.EDIT.equals(portletMode)) {
					return ((PanelSpecificPortlet)this).getPanelEdit();
				}
				else if(PortletMode.HELP.equals(portletMode)) {
					return ((PanelSpecificPortlet)this).getPanelHelp();
				}
			}
			return getMainWindow();
		}

		@Override
		public void handleEventRequest(EventRequest request,
				EventResponse response, Window window) {
			// TODO Maybe something has to done here to delegate treatments to the child class
			initContextEvent();
			log.debug("Logging after initUserContext");
		}

		@Override
		public void handleResourceRequest(ResourceRequest request,
				ResourceResponse response, Window window) {
			initContextEvent();
			log.debug("Logging handleResourceRequest");
		}
	}

}
